% policy_improvement.m:  policy improvement step for policy iteration  
%                        algorithm to solve the machine replacement problem
%
%                        Copyright, All rights reserved, John Rust, Georgetown University, John McClelland, ARA  June 2018
%
% Inputs:
%
% v:  value function (nx1 vector)
%
% Outputs:
%
%     d:  nx1 decision rule (implicitly passed/returned as a global)
%    vl:  nx1 value function (from one contraction step/application of Bellman operator on v)
%     p:  nxn controlled transition probability matrix (implicitly passed/returned as a global)
%    pf:  nx1 profit function evaluated at improved decision rule 
%
% Implicitly passed globals (set in timber.m):
%
%       n:  number of grid points and dimension of v
%     bet:  discount factor 


function [vl,pf]=policy_improvement(v);

global d p n na nm bet oec trc trd rr mtp_pc pr erms;
global notax accounting_life loan_duration borrowing_rate tax_rate debt_financing interest_deductible ipf book_value nodepreciation;
global purchase_price resale_price;

savdat=[];

pf=zeros(n,1);
vl=zeros(n,1);
mstp=zeros(nm,nm);
p=0*p;


% calculate average initial purchase price of machine

for month=1:12;       % loop over month of the year

  i1=nm*(month-1)+1;
  i2=i1+nm-1;
  tpm=mtp_pc(i1:i2,:);        
  monthp=nextmonth(month);

  for a=1:na;         % loop over machine ages

   %fprintf('a=%i month %i month_purchased=%i purchase_price=%g resale_price=%g\n',a,month,month_purchased,purchase_price,resale_price);

    if (a == na);
       ap=na;
    else;
       ap=a+1;
    end;
    ind=nm*na*(monthp-1)+nm*(ap-1)+1;
    vksect=v(ind:ind+nm-1);
    ind=nm*na*(monthp-1)+1;
    v0sect=v(ind:ind+nm-1);

    for ms=1:nm;      % loop over macro state

        sind=nm*na*(month-1)+nm*(a-1)+ms;

        vda=0;  % stores the values for each replacement decision
        pfda=0; % stores the current period profit/loss for each replacement decision

        pfda=erentrev(a,ms,month)-cmaint(a,ms,month);    % profit/loss of keeping current machine
                                                         % before interest and taxes
        interest_payment=0;
        debt_payment=0;
        depr=0;
        tax=0;

        if (debt_financing);
          if (a > loan_duration);     
            interest_payment=0;
            debt_payment=0;
          else;
            interest_payment=debt_financing*purchase_price*ipf(a,3);
            debt_payment=debt_financing*purchase_price*ipf(a,2);
          end;
        else;
            interest_payment=0;
            debt_payment=0;
        end;

        if (~notax);
          if (~nodepreciation);
              if (a == 1);
                depr=(purchase_price-resale_price)*(1-book_value(a));
              elseif (a <= accounting_life);
                depr=(purchase_price-resale_price)*(book_value(a-1)-book_value(a));
              else;
                depr=0;
              end;
          end;

          % calculate EBT (Earnings before taxes)
          if (interest_deductible);
            accounting_profit=pfda-interest_payment-depr;
          else;
            accounting_profit=pfda-depr;
          end;

          % compute tax 
          tax=tax_rate*accounting_profit;

        else;
          tax=0;
        end;

        pfda=pfda-debt_payment-tax;     % this is after tax cash flow, which is the relevant payoff
                                        % that the firm is interested in maximizing, not accounting profit
                                        % which is only useful for determination of taxes

        vk=pfda+bet*tpm(ms,:)*vksect;  % total value of keeping current machine

        vd(a)=vk;

if (pr);
  fprintf(' vk=%g rentrev=%g maint=%g  profit: %g\n',vk,erentrev(a,ms,month),cmaint(a,ms,month),pfda);
end;
         
        pf01=trd*erentrev(0,ms,month)-trc-cmaint(0,ms,month);;
        transcost=oecsales(a,ms,month)-oecsales(0,ms,month);
        tax=0;

        if (debt_financing);
           pf0=transcost+pf01+debt_financing*purchase_price; % amount borrowed for new machine is a cash inflow
           if (a <= loan_duration);
            pf0=pf0-debt_financing*purchase_price*ipf(a,1);  % pay off remaining loan balance on machine that
                                                             % was sold
           end;
        else;
           pf0=transcost+pf01;  % value of replacing with a new machine
        end;

        if (~notax);
          if (~nodepreciation);
             tax=tax_rate*pf01;
          else;
             tax=tax_rate*(pf01+transcost);
          end;
        else;
          tax=0;
        end;

        pf0=pf0-tax;

        v0=pf0+bet*tpm(ms,:)*v0sect;                                   % value of replacing with a new machine

        aopt=a;
        vopt=vk;
        pfopt=pfda;
if (pr);
  fprintf(' v0=%g rentrev=%g maint=%g sales of old machine: %g cost of new machine: %g trc: %g  profit: %g\n',...
       v0,trd*erentrev(0,ms,month),cmaint(0,ms,month),oecsales(a,ms,month),oecsales(0,ms,month),trc,pf0);
end;

        if (v0 > vk);

           aopt=0;
           vopt=v0;
           pfopt=pf0;

        end;

if (pr);
        fprintf('age=%i ms=%i month=%i aopt=%i pfopt=%g vopt=%g profit (keep) %g profit (trade for new) %g vfk=%g vf0=%g\n',...
        a,ms,month,aopt,pfopt,vopt,pf0,pfda,vk,v0);
end;

        if (rr <= 0 && a > 1);  % if no restriction on replacement policy, compute the value of replacing current machine with
                                % used ones of other ages. If rr == 0, only machines younger than current machine are considered,
                                % whereas if rr < 0, then machines older than the current machine are also considered as possible
                                % replacement choices
            pfd=zeros(a-1,1);
            vd=zeros(a-1,1);

            for pa=1:a-1;
      
              ind=nm*na*(monthp-1)+nm*pa+1;
              vsect=v(ind:ind+nm-1);
              pfd(pa)=oecsales(a,ms,month)-oecsales(pa,ms,month)-cmaint(pa,ms,month)-trc+trd*erentrev(pa,ms,month);
              vd(pa)=pfd(pa)+bet*tpm(ms,:)*vsect;

              if (vd(pa) > vopt);
                 vopt=vd(pa);
                 aopt=pa;
                 pfopt=pfd(pa);
              end;

            end;

            if (rr < 0);

              for pa=a+1:-rr;

                if (pa == na);
                   ind=nm*na*(monthp-1)+1;
                else;
                   ind=nm*na*(monthp-1)+nm*pa+1;
                end;
                vsect=v(ind:ind+nm-1);
                pfd(pa)=oecsales(a,ms,month)-oecsales(pa,ms,month)-cmaint(pa,ms,month)-trc+trd*erentrev(pa,ms,month);
                vd(pa)=pfd(pa)+bet*tpm(ms,:)*vsect;

                if (vd(pa) > vopt);
                    vopt=vd(pa);
                    aopt=pa;
                    pfopt=pfd(pa);
                end;

              end;

            end;

            d(sind)=aopt;
            vl(sind)=vopt;
            pf(sind)=pfopt;
            if (aopt == 0);
             nind=(monthp-1)*na*nm+1;
             nind1=nind+nm-1;
             p(sind,nind:nind1)=tpm(ms,:);
            else;
             if (aopt == na);
                aopt=na-1;
             end;
             nind=(monthp-1)*na*nm+aopt*nm+1;
             nind1=nind+nm-1;
             p(sind,nind:nind1)=tpm(ms,:);
            end;

        else;

           if (v0 > vk);

             d(sind)=0;
             vl(sind)=v0;
             pf(sind)=pf0;
             nind=(monthp-1)*na*nm+1;
             nind1=nind+nm-1;
             p(sind,nind:nind1)=tpm(ms,:);

           else;

             d(sind)=a;
             vl(sind)=vk;
             pf(sind)=pfda;
             nind=(monthp-1)*na*nm+(ap-1)*nm+1;
             nind1=nind+nm-1;
             p(sind,nind:nind1)=tpm(ms,:);

           end;

        end;

    end;

  end;

end;

